package org.eclipse.riena.ui.ridgets.tree;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.riena.ui.ridgets.util.beans.AbstractBean;


public class TreeNode extends AbstractBean implements ITreeNode {

	private static final long serialVersionUID = 1L;

	private static void addToParent(ITreeNode parent, ITreeNode child) {
		Assert.isNotNull(child);
		List<ITreeNode> pChildren = parent.getChildren();
		pChildren.add(child);
		parent.setChildren(pChildren);
	}
	private final ITreeNode parent;

	private Object value;

	private List<ITreeNode> children;
	
	public TreeNode(ITreeNode parent, Object value) {
		Assert.isNotNull(value);
		this.parent = parent;
		this.value = value;
		if(parent != null) {
			addToParent(parent, this);
		}
	}
	
	public TreeNode(Object value) {
		this(null, value);
	}

	@Override
	public List<ITreeNode> getChildren() {
		return children != null ? new ArrayList<ITreeNode>(children) : new ArrayList<ITreeNode>();
	}

	@Override
	public ITreeNode getParent() {
		return parent;
	}
	
	@Override
	public Object getValue() {
		return value;
	}
	
	@Override
	public void setChildren(List<ITreeNode> children) {
		if(this.children != children) {
			firePropertyChanged(PROP_CHILDREN, this.children, this.children = new ArrayList<ITreeNode>(children));
		}
	}

	@Override
	public void setValue(Object value) {
		if(this.value != value) {
			Object oldValue = this.value;
			this.value = value;
			firePropertyChanged(PROP_VALUE, oldValue, this.value);
		}
	}

}
