package org.eclipse.riena.ui.ridgets.tree;

import java.util.Collection;
import java.util.List;

import org.eclipse.core.runtime.Assert;

public final class TreeNodeUtils {
	
	public static boolean isLeaf(ITreeNode node) {
		Collection<ITreeNode> children = node.getChildren();
		return children.size() == 0;
	}
	
	public static int getLevel(ITreeNode node) {
		int result = 0;
		ITreeNode parent = node.getParent();
		while(parent != null) {
			result++;
			parent = parent.getParent();
		}
		return result;
	}
	
	public static int getChildCount(ITreeNode node) {
		return node.getChildren().size();
	}
	
	public static int getIndex(ITreeNode parent, ITreeNode child) {
		List<ITreeNode> children = parent.getChildren();
		return children.indexOf(child);
	}
	
	public static ITreeNode getChild(ITreeNode parent, int index) {
		Assert.isLegal(index >= 0);
		ITreeNode result = null;
		List<ITreeNode> children = parent.getChildren();
		if (index < children.size()) {
			result = (ITreeNode) children.get(index);
		}
		return result;
	}
	
	public static ITreeNode getRoot(ITreeNode node) {
		ITreeNode result = node;
		while(result.getParent() != null) {
			result = result.getParent();
		}
		return result;
	}
	
}
