package test;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.value.ComputedValue;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.databinding.viewers.ObservableSetTreeContentProvider;
import org.eclipse.jface.databinding.viewers.ViewersObservables;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.riena.ui.ridgets.util.beans.StringBean;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

public class Snippet020StringBean {

	/* bean property to use for storing children */
	private static final String PROP_ELEM_SET = "children";

	/* bean property to use for labels */
	private static final String PROP_ELEM_VALUE = "value";
	
	private Button pasteButton;
	private Button copyButton;
	private Shell shell;
	private Button addChildBeanButton;
	private Button removeBeanButton;
	private TreeViewer beanViewer;
	private Tree tree;
	private Text beanText;
	private DataBindingContext m_bindingContext;

	private StringBean input = createBean("input");
	private IObservableValue clipboard;
	static int counter = 0;

	/**
	 * Launch the application
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		Display display = Display.getDefault();
		Realm.runWithDefault(SWTObservables.getRealm(display), new Runnable() {
			public void run() {
				try {
					Snippet020StringBean window = new Snippet020StringBean();
					window.open();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Open the window
	 */
	public void open() {
		final Display display = Display.getDefault();
		createContents();
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * Create contents of the window
	 */
	protected void createContents() {
		shell = new Shell();
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		shell.setLayout(gridLayout_1);
		shell.setSize(535, 397);
		shell.setText("SWT Application (StringBean)");

		final Composite group = new Composite(shell, SWT.NONE);
		final RowLayout rowLayout = new RowLayout();
		rowLayout.marginTop = 0;
		rowLayout.marginRight = 0;
		rowLayout.marginLeft = 0;
		rowLayout.marginBottom = 0;
		rowLayout.pack = false;
		group.setLayout(rowLayout);
		group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 2, 1));

		final Button addRootButton = new Button(group, SWT.NONE);
		addRootButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				Collection<StringBean> children = input.getChildren();
				StringBean root = createBean("root");
				children.add(root);
				input.setChildren(children);

				beanViewer.setSelection(new StructuredSelection(root));
				beanText.selectAll();
				beanText.setFocus();
			}
		});
		addRootButton.setText("Add Root");

		addChildBeanButton = new Button(group, SWT.NONE);
		addChildBeanButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				StringBean parent = getSelectedBean();
				Collection<StringBean> children = new ArrayList<StringBean>(parent.getChildren());
				StringBean child = createBean("child" + (counter++));
				children.add(child);
				parent.setChildren(children);
			}
		});
		addChildBeanButton.setText("Add Child");

		removeBeanButton = new Button(group, SWT.NONE);
		removeBeanButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				TreeItem selectedItem = beanViewer.getTree().getSelection()[0];
				Object bean = selectedItem.getData();
				TreeItem parentItem = selectedItem.getParentItem();
				StringBean parent;
				if (parentItem == null)
					parent = input;
				else
					parent = (StringBean) parentItem.getData();
				
				Collection<StringBean> children = new ArrayList<StringBean>(parent.getChildren());
				children.remove(bean);
				parent.setChildren(children);
			}
		});
		removeBeanButton.setText("Remove");

		copyButton = new Button(group, SWT.NONE);
		copyButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				clipboard.setValue(getSelectedBean());
			}
		});
		copyButton.setText("Copy");

		pasteButton = new Button(group, SWT.NONE);
		pasteButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				StringBean copy = (StringBean) clipboard.getValue();
				if (copy == null)
					return;
				StringBean parent = getSelectedBean();
				if (parent == null)
					parent = input;

				Collection<StringBean> children = new ArrayList<StringBean>(parent.getChildren());
				children.add(copy);
				parent.setChildren(children);

				beanViewer.setSelection(new StructuredSelection(copy));
				beanText.selectAll();
				beanText.setFocus();
			}
		});
		pasteButton.setText("Paste");

		final Button refreshButton = new Button(group, SWT.NONE);
		refreshButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				beanViewer.refresh();
			}
		});
		refreshButton.setText("Refresh");

		beanViewer = new TreeViewer(shell, SWT.FULL_SELECTION | SWT.BORDER);
		beanViewer.setUseHashlookup(true);
		beanViewer.setComparator(new ViewerComparator());
		tree = beanViewer.getTree();
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));

		final Label itemNameLabel = new Label(shell, SWT.NONE);
		itemNameLabel.setText("Item Name");

		beanText = new Text(shell, SWT.BORDER);
		final GridData gd_beanValue = new GridData(SWT.FILL, SWT.CENTER, true,
				false);
		beanText.setLayoutData(gd_beanValue);
		m_bindingContext = initDataBindings();
		
		initExtraBindings(m_bindingContext);
	}

	private static StringBean createBean(String name) {
		return new StringBean(name);
	}

	protected DataBindingContext initDataBindings() {
		// binds the value of the selected bean to a text field
		IObservableValue treeViewerSelectionObserveSelection = ViewersObservables
				.observeSingleSelection(beanViewer);
		IObservableValue textTextObserveWidget = SWTObservables.observeText(
				beanText, SWT.Modify);
		IObservableValue treeViewerValueObserveDetailValue = BeansObservables
				.observeDetailValue(Realm.getDefault(),
						treeViewerSelectionObserveSelection, PROP_ELEM_VALUE,
						java.lang.String.class);

		DataBindingContext bindingContext = new DataBindingContext();
		bindingContext.bindValue(textTextObserveWidget,
				treeViewerValueObserveDetailValue, null, null);
		return bindingContext;
	}

	private StringBean getSelectedBean() {
		IStructuredSelection selection = (IStructuredSelection) beanViewer
				.getSelection();
		if (selection.isEmpty())
			return null;
		return (StringBean) selection.getFirstElement();
	}

	private void initExtraBindings(DataBindingContext dbc) {
		// data bindings for button enablement / disablement
		final IObservableValue beanViewerSelection = ViewersObservables
				.observeSingleSelection(beanViewer);
		IObservableValue beanSelected = new ComputedValue(Boolean.TYPE) {
			protected Object calculate() {
				return Boolean.valueOf(beanViewerSelection.getValue() != null);
			}
		};
		dbc.bindValue(SWTObservables.observeEnabled(addChildBeanButton),
				beanSelected, null, null);
		dbc.bindValue(SWTObservables.observeEnabled(removeBeanButton),
				beanSelected, null, null);

		clipboard = new WritableValue();
		dbc.bindValue(SWTObservables.observeEnabled(copyButton),
				beanSelected, null, null);
		dbc.bindValue(SWTObservables.observeEnabled(pasteButton), new ComputedValue(Boolean.TYPE) {
			protected Object calculate() {
				return Boolean.valueOf(clipboard.getValue() != null);
			}
		}, null, null);

		
		// data binding for the tree
		ObservableSetTreeContentProvider contentProvider = new ObservableSetTreeContentProvider(
				BeansObservables.setFactory(Realm.getDefault(), PROP_ELEM_SET,
						StringBean.class), null);
		beanViewer.setContentProvider(contentProvider);
		beanViewer.setLabelProvider(new ObservableMapLabelProvider(
				BeansObservables.observeMap(contentProvider.getKnownElements(),
						StringBean.class, PROP_ELEM_VALUE)));
		beanViewer.setInput(input);
	}

}