package test;

import java.util.List;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.value.ComputedValue;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.databinding.viewers.ObservableSetTreeContentProvider;
import org.eclipse.jface.databinding.viewers.ViewersObservables;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.riena.ui.ridgets.tree.ITreeNode;
import org.eclipse.riena.ui.ridgets.tree.TreeNode;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;

public class Snippet020TreeNode {

	private Button pasteButton;
	private Button copyButton;
	private Shell shell;
	private Button addChildBeanButton;
	private Button removeBeanButton;
	private TreeViewer beanViewer;
	private Tree tree;
	private Text beanText;
	private DataBindingContext m_bindingContext;

	private ITreeNode input = createTreeModel();
	private IObservableValue clipboard;
	static int counter = 0;

	/**
	 * Launch the application
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		Display display = Display.getDefault();
		Realm.runWithDefault(SWTObservables.getRealm(display), new Runnable() {
			public void run() {
				try {
					Snippet020TreeNode window = new Snippet020TreeNode();
					window.open();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Open the window
	 */
	public void open() {
		final Display display = Display.getDefault();
		createContents();
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * Create contents of the window
	 */
	protected void createContents() {
		shell = new Shell();
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		shell.setLayout(gridLayout_1);
		shell.setSize(535, 397);
		shell.setText("SWT Application (TreeNode)");

		final Composite group = new Composite(shell, SWT.NONE);
		final RowLayout rowLayout = new RowLayout();
		rowLayout.marginTop = 0;
		rowLayout.marginRight = 0;
		rowLayout.marginLeft = 0;
		rowLayout.marginBottom = 0;
		rowLayout.pack = false;
		group.setLayout(rowLayout);
		group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 2, 1));

		final Button addRootButton = new Button(group, SWT.NONE);
		addRootButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				TreeNode root = new TreeNode(input, "root");
				beanViewer.setSelection(new StructuredSelection(root));
				beanText.selectAll();
				beanText.setFocus();
			}
		});
		addRootButton.setText("Add Root");

		addChildBeanButton = new Button(group, SWT.NONE);
		addChildBeanButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				ITreeNode parent = getSelectedBean();
				new TreeNode(parent, "Child " + (counter++));
			}
		});
		addChildBeanButton.setText("Add Child");

		removeBeanButton = new Button(group, SWT.NONE);
		removeBeanButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				ITreeNode node = getSelectedBean();
				ITreeNode parent = node.getParent();
				if(parent == null) {
					parent = input;
				}
				List<ITreeNode> children = parent.getChildren();
				children.remove(node);
				parent.setChildren(children);
			}
		});
		removeBeanButton.setText("Remove");

		copyButton = new Button(group, SWT.NONE);
		copyButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				clipboard.setValue(getSelectedBean());
			}
		});
		copyButton.setText("Copy");

		pasteButton = new Button(group, SWT.NONE);
		pasteButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				ITreeNode copy = (ITreeNode) clipboard.getValue();
				if (copy == null)
					return;
				ITreeNode parent = getSelectedBean();
				if (parent == null)
					parent = input;

				List<ITreeNode> children = parent.getChildren();
				children.add(copy);
				parent.setChildren(children);

				beanViewer.setSelection(new StructuredSelection(copy));
				beanText.selectAll();
				beanText.setFocus();
			}
		});
		pasteButton.setText("Paste");

		final Button refreshButton = new Button(group, SWT.NONE);
		refreshButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				beanViewer.refresh();
			}
		});
		refreshButton.setText("Refresh");

		beanViewer = new TreeViewer(shell, SWT.FULL_SELECTION | SWT.BORDER);
		beanViewer.setUseHashlookup(true);
		beanViewer.setComparator(new ViewerComparator());
		tree = beanViewer.getTree();
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));

		final Label itemNameLabel = new Label(shell, SWT.NONE);
		itemNameLabel.setText("Item Name");

		beanText = new Text(shell, SWT.BORDER);
		final GridData gd_beanValue = new GridData(SWT.FILL, SWT.CENTER, true,
				false);
		beanText.setLayoutData(gd_beanValue);
		m_bindingContext = initDataBindings();
		
		initExtraBindings(m_bindingContext);
	}

	protected DataBindingContext initDataBindings() {
		// binds the value of the selected bean to a text field
		IObservableValue treeViewerSelectionObserveSelection = ViewersObservables
				.observeSingleSelection(beanViewer);
		IObservableValue textTextObserveWidget = SWTObservables.observeText(
				beanText, SWT.Modify);
		IObservableValue treeViewerValueObserveDetailValue = BeansObservables
				.observeDetailValue(Realm.getDefault(),
						treeViewerSelectionObserveSelection, ITreeNode.PROP_VALUE, Object.class);
		DataBindingContext bindingContext = new DataBindingContext();
		bindingContext.bindValue(textTextObserveWidget, treeViewerValueObserveDetailValue, null, null);
		return bindingContext;
	}

	private ITreeNode getSelectedBean() {
		IStructuredSelection selection = (IStructuredSelection) beanViewer
				.getSelection();
		if (selection.isEmpty())
			return null;
		return (ITreeNode) selection.getFirstElement();
	}

	private void initExtraBindings(DataBindingContext dbc) {
		// data bindings for button enablement / disablement
		final IObservableValue beanViewerSelection = ViewersObservables
				.observeSingleSelection(beanViewer);
		IObservableValue beanSelected = new ComputedValue(Boolean.TYPE) {
			protected Object calculate() {
				return Boolean.valueOf(beanViewerSelection.getValue() != null);
			}
		};
		dbc.bindValue(SWTObservables.observeEnabled(addChildBeanButton),
				beanSelected, null, null);
		dbc.bindValue(SWTObservables.observeEnabled(removeBeanButton),
				beanSelected, null, null);

		clipboard = new WritableValue();
		dbc.bindValue(SWTObservables.observeEnabled(copyButton),
				beanSelected, null, null);
		dbc.bindValue(SWTObservables.observeEnabled(pasteButton), new ComputedValue(Boolean.TYPE) {
			protected Object calculate() {
				return Boolean.valueOf(clipboard.getValue() != null);
			}
		}, null, null);

		
		// data binding for the tree
		ObservableSetTreeContentProvider contentProvider = new ObservableSetTreeContentProvider(
				BeansObservables.setFactory(Realm.getDefault(), ITreeNode.PROP_CHILDREN,
						ITreeNode.class), null);
		beanViewer.setContentProvider(contentProvider);
		beanViewer.setLabelProvider(new ObservableMapLabelProvider(
				BeansObservables.observeMap(contentProvider.getKnownElements(),
						ITreeNode.class, ITreeNode.PROP_VALUE)));
		beanViewer.setInput(input);
	}
	
	private static ITreeNode createTreeModel() {
		ITreeNode root = new TreeNode("root");

		ITreeNode groupA = new TreeNode(root, "group a");
		new TreeNode(groupA, "a_child_1");
		new TreeNode(groupA, "a_child_2");
		new TreeNode(groupA, "a_child_3");

		ITreeNode groupB = new TreeNode(root, "group b");
		new TreeNode(groupB, "b_child_1");
		new TreeNode(groupB, "b_child_2");
		new TreeNode(groupB, "b_child_3");

		ITreeNode groupC = new TreeNode(root, "group c");
		new TreeNode(groupC, "c_child_1");
		new TreeNode(groupC, "c_child_2");
		new TreeNode(groupC, "c_child_3");

		return root;
	}


}